package ru.dbotthepony.kstarbound.gl

import org.lwjgl.opengl.GL46.*
import java.io.Closeable
import java.nio.ByteBuffer

enum class VBOType(val value: Int) {
	ARRAY(GL_ARRAY_BUFFER),
	ELEMENT_ARRAY(GL_ELEMENT_ARRAY_BUFFER),
}

class GLVertexBufferObject(val state: GLStateTracker, val type: VBOType = VBOType.ARRAY) : Closeable {
	val pointer = glGenBuffers()

	val isArray get() = type == VBOType.ARRAY
	val isElementArray get() = type == VBOType.ELEMENT_ARRAY

	fun bind(): GLVertexBufferObject {
		check(isValid) { "Tried to use NULL GLVertexBufferObject" }
		state.bind(this)
		return this
	}

	fun unbind(): GLVertexBufferObject {
		check(isValid) { "Tried to use NULL GLVertexBufferObject" }
		state.unbind(this)
		return this
	}

	fun bufferData(data: ByteBuffer, usage: Int): GLVertexBufferObject {
		check(isValid) { "Tried to use NULL GLVertexBufferObject" }
		state.ensureSameThread()
		glNamedBufferData(pointer, data, usage)
		checkForGLError()
		return this
	}

	fun bufferData(data: IntArray, usage: Int): GLVertexBufferObject {
		check(isValid) { "Tried to use NULL GLVertexBufferObject" }
		state.ensureSameThread()
		glNamedBufferData(pointer, data, usage)
		checkForGLError()
		return this
	}

	fun bufferData(data: FloatArray, usage: Int): GLVertexBufferObject {
		check(isValid) { "Tried to use NULL GLVertexBufferObject" }
		state.ensureSameThread()
		glNamedBufferData(pointer, data, usage)
		checkForGLError()
		return this
	}

	fun bufferData(data: DoubleArray, usage: Int): GLVertexBufferObject {
		check(isValid) { "Tried to use NULL GLVertexBufferObject" }
		state.ensureSameThread()
		glNamedBufferData(pointer, data, usage)
		checkForGLError()
		return this
	}

	fun bufferData(data: LongArray, usage: Int): GLVertexBufferObject {
		check(isValid) { "Tried to use NULL GLVertexBufferObject" }
		state.ensureSameThread()
		glNamedBufferData(pointer, data, usage)
		checkForGLError()
		return this
	}

	var isValid = true
		private set

	override fun close() {
		state.ensureSameThread()
		if (!isValid) return

		if (state.VBO == this) {
			state.VBO = null
		}

		glDeleteBuffers(pointer)
		isValid = false
	}
}
